# Generated by Django 3.1.2 on 2021-05-15 03:37

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0014_auto_20210105_1413'),
        ('moonstuff', '0004_delete_trackingcharacter'),
    ]

    operations = [
        migrations.CreateModel(
            name='TrackingCharacter',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('latest_notification_id', models.BigIntegerField(default=0, null=True)),
                ('last_notification_check', models.DateTimeField(null=True)),
                ('character', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='eveonline.evecharacter')),
            ],
            options={
                'default_permissions': ('add',),
            },
        ),
    ]
