# Generated by Django 3.1.2 on 2021-06-17 02:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('moonstuff', '0005_trackingcharacter'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='refinery',
            options={'default_permissions': (), 'verbose_name_plural': 'Refineries'},
        ),
    ]
