{% load i18n %}

<nav class="navbar navbar-default">
    <div class="container-fluid">
        <div class="navbar-header">
            <button type="button" class="navbar-toggle collapsed" data-toggle="collapse" data-target="#bs-example-navbar-collapse-1" aria-expanded="false">
                <span class="sr-only">{% translate "Toggle navigation" %}</span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
            </button>
            <a class="navbar-brand" href="{% url 'moonstuff:dashboard' %}">{% translate "Moons" %}</a>
        </div>
         {% if perms.moonstuff.add_trackingcharacter %}
            <div class="dropdown pull-right" style="margin-top: .8em;">
                <button class="btn btn-success btn-sm dropdown-toggle" type="button" id="manageDrop" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
                    <span class="fa fa-plus"></span>
                </button>
                <ul class="dropdown-menu" aria-labelledby="manageDrop">
                    <li> <a href="{% url 'moonstuff:add_scan' %}">{% translate "Moon Scan" %}</a></li>
                    <li> <a href="{% url 'moonstuff:add_character' %}">{% translate "Tracking Character" %}</a></li>
                </ul>
            </div>
        {% endif %}
    </div>
</nav>