# Generated by Django 3.1.2 on 2021-05-15 03:22

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('moonstuff', '0003_OtO_TrackingCharacter_Character'),
    ]

    operations = [
        migrations.DeleteModel(
            name='TrackingCharacter',
        ),
    ]
