# Generated by Django 3.1.2 on 2021-04-19 06:44

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('moonstuff', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='material',
            name='evetype',
        ),
        migrations.RemoveField(
            model_name='material',
            name='material_evetype',
        ),
        migrations.DeleteModel(
            name='MaterialCheckSum',
        ),
        migrations.DeleteModel(
            name='Material',
        ),
    ]
