:root {
    --R0: #9f3936;
    --R4: #b09635;
    --R8: #40889b;
    --R16: #0cc187;
    --R32: #0aab11;
    --R64: #801ede;
    --ext: #c87936;
}

.panel-ext {
    border-radius: 0;
    border: 0;
}
.panel-ext .panel-heading {
    border-radius: 0;
    background-color: var(--ext);
    color: white;
}
.template-light-mode .panel-ext .panel-body {
    background: #ECF0F1;
}
.template-dark-mode .panel-ext .panel-body {
    background: #383838;
}
.progress{
    background: #bfbfbf;
    margin-left: 20%;
    margin-right: 20%;
    margin-bottom: 0;
    height: 20px;
}
.flex-container {
    display: flex;
    flex-direction: row;
    flex-wrap: wrap;
    justify-content: space-evenly;
    align-items: flex-start;
    align-content: space-around;
}
.fc-lbl {
    margin: 1px !important;
    float: none !important;
}
.modal-content .bg-danger {
    border-bottom: 0;
}
.bgd {
    background-color: rgba(128, 128, 128, .65);
}
.bg-R0 {
    background-color: var(--R0);
}
.bg-R4 {
    background-color: var(--R4);
}
.bg-R8 {
    background-color: var(--R8);
}
.bg-R16 {
    background-color: var(--R16);
}
.bg-R32 {
    background-color: var(--R32);
}
.bg-R64 {
    background-color: var(--R64);
}
.percent {
    padding-top: 2px;
    padding-left: 2px;
}
a.info-link {
    text-decoration: inherit;
    color: inherit;
}

.ribbon {
  position: absolute;
  left: 10px; top: -5px;
  z-index: 1;
  overflow: hidden;
  width: 75px; height: 75px;
  text-align: right;
}
.ribbon span {
  font-size: 10px;
  font-weight: bold;
  color: #FFF;
  text-transform: uppercase;
  text-align: center;
  line-height: 20px;
  transform: rotate(-45deg);
  -webkit-transform: rotate(-45deg);
  width: 100px;
  display: block;
  background: #79A70A;
  background: linear-gradient(#C9BC00 0%, #A19600 100%);
  box-shadow: 0 3px 10px -5px rgba(0, 0, 0, 1);
  position: absolute;
  top: 19px; left: -21px;
}
.ribbon span::before {
  content: "";
  position: absolute; left: 0px; top: 100%;
  z-index: -1;
  border-left: 3px solid #C9BC00;
  border-right: 3px solid transparent;
  border-bottom: 3px solid transparent;
  border-top: 3px solid #C9BC00;
}
.ribbon span::after {
  content: "";
  position: absolute; right: 0px; top: 100%;
  z-index: -1;
  border-left: 3px solid transparent;
  border-right: 3px solid #C9BC00;
  border-bottom: 3px solid transparent;
  border-top: 3px solid #C9BC00;
}