# Generated by Django 3.1.2 on 2021-04-20 06:20

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0014_auto_20210105_1413'),
        ('moonstuff', '0002_remove_material_model'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='trackingcharacter',
            name='id',
        ),
        migrations.AlterField(
            model_name='trackingcharacter',
            name='character',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True, serialize=False, to='eveonline.evecharacter'),
        ),
    ]
